
#include	"libExt.h"
#include	"Torus.h"

///////////////////////////////////////////////////////////////////////////////////

Torus :: Torus ( float r1, float r2, int rings, int sides )
{
	float 	ringDelta = 2.0 * M_PI / rings;
	float 	sideDelta = 2.0 * M_PI / sides;
	float	invRings  = 1.0f / (float) rings;
	float	invSides  = 1.0f / (float) sides;
	int 	i, j;

	vao      = NULL;
	numRings = rings;
	numSides = sides;
	vertices = new Vertex [(numSides+1)*(numRings+1)];
	faces    = new Face   [numSides*numRings*2];

								// create vertices
	int	index = 0;

	for ( i = 0; i <= rings; i++ )
	{
    	float theta    = i * ringDelta;
	    float cosTheta = cos ( theta );
    	float sinTheta = sin ( theta );

		for ( j = 0; j <= sides; j++ )
		{
			float phi    = j * sideDelta;
			float cosPhi = cos ( phi );
			float sinPhi = sin ( phi );
			float dist = r2 + r1 * cosPhi;

      	// (x,y,z) below is the parametric equation for a torus
        // when theta and phi both vary from 0 to pi.

      				// x = cos(theta) * (R + r * cos(phi))
      				// y = -sin(theta) * (R + r * cos(phi))
      				// z = r * sin(phi)

      		vertices [index].pos.x = cosTheta * dist;
      		vertices [index].pos.y = -sinTheta * dist;
      		vertices [index].pos.z = r1 * sinPhi;

      // now setup texture coordinates

			vertices [index].tex.x = j * invSides;
      		vertices [index].tex.y = i * invRings;

		// compute unit normal

			vertices [index].n.x =  cosTheta * cosPhi;
			vertices [index].n.y = -sinTheta * cosPhi;
			vertices [index].n.z = sinPhi;

      		index++;
		}
	}

	numVertices = index;

											// Create faces
	index = 0;

	for ( i = 0; i < rings; i++ )
		for ( j = 0; j < sides; j++, index += 2 )
		{
			faces [index].index [0] = i*(sides+1) + j;
			faces [index].index [1] = (i+1)*(sides+1) + j;
			faces [index].index [2] = (i+1)*(sides+1) + j + 1;

			faces [index+1].index [0] = i*(sides+1) + j;
			faces [index+1].index [1] = (i+1)*(sides+1) + j + 1;
			faces [index+1].index [2] = i*(sides+1) + j + 1;
		}

	numFaces = index;

	vertexId = 0;							// no one yet
	indexId  = 0;
}

Torus :: ~Torus ()
{
	delete    vao;
	delete [] vertices;
	delete [] faces;

	glDeleteBuffersARB ( 1, &vertexId );
	glDeleteBuffersARB ( 1, &indexId  );
}

void	Torus :: setupBuffers ()
{
											// create vertex data VBO
	glGenBuffersARB ( 1, &vertexId );
	glBindBufferARB ( GL_ARRAY_BUFFER_ARB, vertexId );
	glBufferDataARB ( GL_ARRAY_BUFFER_ARB, numVertices * sizeof ( Vertex ), vertices, GL_STATIC_DRAW_ARB );

											// create index data VBO
	glGenBuffersARB ( 1, &indexId );
	glBindBufferARB ( GL_ELEMENT_ARRAY_BUFFER_ARB, indexId );
	glBufferDataARB ( GL_ELEMENT_ARRAY_BUFFER_ARB, numFaces * sizeof ( Face ), faces, GL_STATIC_DRAW_ARB );

											// unbind buffers
	int	vertexStride = sizeof ( Vertex );
	int	texOffs      = ((int)&vertices [0].tex) - ((int)&vertices [0]);

	vao = new VertexArray ();

	vao -> bind ();

	glPushClientAttrib  ( GL_CLIENT_VERTEX_ARRAY_BIT );
	glEnableClientState ( GL_VERTEX_ARRAY );
	glEnableClientState ( GL_TEXTURE_COORD_ARRAY );

	glBindBufferARB   ( GL_ARRAY_BUFFER_ARB, vertexId );
	glVertexPointer   ( 3, GL_FLOAT, vertexStride, (void *)0 );
	glTexCoordPointer ( 2, GL_FLOAT, vertexStride, (void *)texOffs );

	glBindBufferARB   ( GL_ELEMENT_ARRAY_BUFFER_ARB, indexId );

	vao -> unbind ();

}

void	Torus :: draw ()
{
	vao -> bind ();

	glDrawElements    ( GL_TRIANGLES, 3*numFaces, GL_UNSIGNED_INT, 0 );

	vao -> unbind ();

}

//////////////////////////// end of torus methods //////////////////////////////////
